// Class: haxe.format.JsonParser

var $global = typeof window != "undefined" ? window : typeof global != "undefined" ? global : typeof self != "undefined" ? self : this

$global.Object.defineProperty(exports, "__esModule", {value: true});

var __map_reserved = {};

// Imports

var $hxClasses = require("./../../hxClasses_stub").default;
var $import = require("./../../import_stub").default;
function Reflect() {return require("./../../Reflect");}
function StringBuf() {return require("./../../StringBuf");}
function Std() {return require("./../../Std");}
function HxOverrides() {return require("./../../HxOverrides");}
function js__$Boot_HaxeError() {return require("./../../js/_Boot/HaxeError");}
function StringTools() {return require("./../../StringTools");}

// Constructor

var JsonParser = function(str) {
	this.str = str;
	this.pos = 0;
}

// Meta

JsonParser.__name__ = ["haxe","format","JsonParser"];
JsonParser.prototype = {
	parseRec: function() {
		while(true) {
			var c = this.nextChar();
			switch(c) {
			case 9:case 10:case 13:case 32:
				break;
			case 34:
				return this.parseString();
			case 45:case 48:case 49:case 50:case 51:case 52:case 53:case 54:case 55:case 56:case 57:
				return this.parseNumber(c);
			case 91:
				var arr = [];
				var comma = null;
				while(true) {
					var c1 = this.nextChar();
					switch(c1) {
					case 9:case 10:case 13:case 32:
						break;
					case 44:
						if(comma) {
							comma = false;
						} else {
							this.invalidChar();
						}
						break;
					case 93:
						if(comma == false) {
							this.invalidChar();
						}
						return arr;
					default:
						if(comma) {
							this.invalidChar();
						}
						this.pos--;
						arr.push(this.parseRec());
						comma = true;
					}
				}
				break;
			case 102:
				var save = this.pos;
				if(this.nextChar() != 97 || this.nextChar() != 108 || this.nextChar() != 115 || this.nextChar() != 101) {
					this.pos = save;
					this.invalidChar();
				}
				return false;
			case 110:
				var save1 = this.pos;
				if(this.nextChar() != 117 || this.nextChar() != 108 || this.nextChar() != 108) {
					this.pos = save1;
					this.invalidChar();
				}
				return null;
			case 116:
				var save2 = this.pos;
				if(this.nextChar() != 114 || this.nextChar() != 117 || this.nextChar() != 101) {
					this.pos = save2;
					this.invalidChar();
				}
				return true;
			case 123:
				var obj = { };
				var field = null;
				var comma1 = null;
				while(true) {
					var c2 = this.nextChar();
					switch(c2) {
					case 9:case 10:case 13:case 32:
						break;
					case 34:
						if(comma1) {
							this.invalidChar();
						}
						field = this.parseString();
						break;
					case 44:
						if(comma1) {
							comma1 = false;
						} else {
							this.invalidChar();
						}
						break;
					case 58:
						if(field == null) {
							this.invalidChar();
						}
						(Reflect().default).setField(obj,field,this.parseRec());
						field = null;
						comma1 = true;
						break;
					case 125:
						if(field != null || comma1 == false) {
							this.invalidChar();
						}
						return obj;
					default:
						this.invalidChar();
					}
				}
				break;
			default:
				this.invalidChar();
			}
		}
	},
	parseString: function() {
		var start = this.pos;
		var buf = null;
		while(true) {
			var c = this.nextChar();
			if(c == 34) {
				break;
			}
			if(c == 92) {
				if(buf == null) {
					buf = new (StringBuf().default)();
				}
				buf.addSub(this.str,start,this.pos - start - 1);
				c = this.nextChar();
				switch(c) {
				case 34:case 47:case 92:
					buf.addChar(c);
					break;
				case 98:
					buf.addChar(8);
					break;
				case 102:
					buf.addChar(12);
					break;
				case 110:
					buf.addChar(10);
					break;
				case 114:
					buf.addChar(13);
					break;
				case 116:
					buf.addChar(9);
					break;
				case 117:
					var uc = (Std().default).parseInt("0x" + (HxOverrides().default).substr(this.str,this.pos,4));
					this.pos += 4;
					buf.addChar(uc);
					break;
				default:
					throw new (js__$Boot_HaxeError().default)("Invalid escape sequence \\" + String.fromCharCode(c) + " at position " + (this.pos - 1));
				}
				start = this.pos;
			} else if((StringTools().default).isEof(c)) {
				throw new (js__$Boot_HaxeError().default)("Unclosed string");
			}
		}
		if(buf == null) {
			return (HxOverrides().default).substr(this.str,start,this.pos - start - 1);
		} else {
			buf.addSub(this.str,start,this.pos - start - 1);
			return buf.toString();
		}
	},
	parseNumber: function(c) {
		var start = this.pos - 1;
		var minus = c == 45;
		var digit = !minus;
		var zero = c == 48;
		var point = false;
		var e = false;
		var pm = false;
		var end = false;
		while(true) {
			c = this.nextChar();
			switch(c) {
			case 43:case 45:
				if(!e || pm) {
					this.invalidNumber(start);
				}
				digit = false;
				pm = true;
				break;
			case 46:
				if(minus || point) {
					this.invalidNumber(start);
				}
				digit = false;
				point = true;
				break;
			case 48:
				if(zero && !point) {
					this.invalidNumber(start);
				}
				if(minus) {
					minus = false;
					zero = true;
				}
				digit = true;
				break;
			case 49:case 50:case 51:case 52:case 53:case 54:case 55:case 56:case 57:
				if(zero && !point) {
					this.invalidNumber(start);
				}
				if(minus) {
					minus = false;
				}
				digit = true;
				zero = false;
				break;
			case 69:case 101:
				if(minus || zero || e) {
					this.invalidNumber(start);
				}
				digit = false;
				e = true;
				break;
			default:
				if(!digit) {
					this.invalidNumber(start);
				}
				this.pos--;
				end = true;
			}
			if(end) {
				break;
			}
		}
		var f = (Std().default).parseFloat((HxOverrides().default).substr(this.str,start,this.pos - start));
		var i = (Std().default)["int"](f);
		if(i == f) {
			return i;
		} else {
			return f;
		}
	},
	nextChar: function() {
		return (StringTools().default).fastCodeAt(this.str,this.pos++);
	},
	invalidChar: function() {
		this.pos--;
		throw new (js__$Boot_HaxeError().default)("Invalid char " + (StringTools().default).fastCodeAt(this.str,this.pos) + " at position " + this.pos);
	},
	invalidNumber: function(start) {
		throw new (js__$Boot_HaxeError().default)("Invalid number at position " + start + ": " + (HxOverrides().default).substr(this.str,start,this.pos - start));
	}
};
JsonParser.prototype.__class__ = $hxClasses["haxe.format.JsonParser"] = JsonParser;

// Init



// Statics

JsonParser.parse = function(str) {
	return new JsonParser(str).parseRec();
}


// Export

exports.default = JsonParser;